/*-------------------------------------------------------------------------
main.c:
Emulator entry point; options procesing.
-------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#ifdef linux
#include <asm/termios.h>
#else
#include <termios.h>
#endif
#include <sys/time.h>
#include <signal.h>
#include <unistd.h>
#include "pic.h"

// RAM & REGs:
core f84;
// Program:
word prog[1024];
// EEprom:

// Set canonical mode for stdin (line editing)
void setcanon()
{
        struct termios term;
        tcgetattr(0,&term);
        term.c_lflag|=ICANON;
        term.c_lflag|=ECHO;
        if (tcsetattr(0,TCSANOW,&term)) {
                fprintf(stderr,"tcsetattr failed\n");
                exit(0);
        }
}

// Set NOcanonical mode for stdin (single character input)
// gcc-2.95.3(i386) generates buggy code for this function
// (uncontrolled stack pointer)

void setnocanon()
{
        struct termios term;
        tcgetattr(0,&term);
        term.c_lflag&=~ICANON;
        term.c_lflag&=~ECHO;
        // Solaris is a bit stupid fetching 4 chars at a time by default.
        // It tries to be "more efficient". But I think noncanonical mode
        // is used for reading 1 char at a time 99.9(period) % of times
        term.c_cc[VMIN]=1;      // Force 1 char at a time (too smart Sunos)
        term.c_cc[VTIME]=0;
        if (tcsetattr(0,TCSANOW,&term)) {
                fprintf(stderr,"tcsetattr failed\n");
                exit(0);
        }
}

// Crtl-C handler
void crtl_c()
{
        debug_status=STEP;
}

// Alarm handler (no nothing but exit pause();)
void alarmh()
{
}

//-------------------------------------------------------------------------
// Main entry point
//-------------------------------------------------------------------------

int main(int argc, char **argv)
{
        int i,n=0,conf=0;
        const char *osc_types[4]={
        "(LP) Low power/low frequency (<0.1MHz) Crystal  Oscillator",
        "(XT) Medium power/medium frequency (0.1-4MHz) Crystal/Resonator Oscillator",
        "(HS) High power/high Speed (>4MHz) Crystal Oscillator",
        "(RC) Resistor/Capacitor Oscillator" };
        struct itimerval t0,t1;
        struct sigaction act0,act1;
        f84.freq=1000000;

        if (argc<2) {
                fprintf(stderr,"\n%s version %s\n\n",argv[0],VERSION);
                fprintf(stderr,"Use: %s [Options] <file.ihx>\n",argv[0]);
                fprintf(stderr,"Options:\n");
                fprintf(stderr,"-i            Print configuration info and exit\n");
                fprintf(stderr,"-r            Run. Do not enter Debug\n");
                fprintf(stderr,"-c<n_cycles>  Run <n_cycles>, then enter Debug\n");
                fprintf(stderr,"-f<Osc>       Oscillator frequency> (MHz)\n\n");
                exit(0);
        }
        for (i=1;i<argc;i++) {
                if (*argv[i]!='-') loadhex(argv[i],prog, &f84);
                else switch (argv[i][1]) {
                case 'i': conf=1; break;
                case 'r': debug_status=RUNNING; break;
                case 'c': n=atoi(&argv[i][2]); break;
                case 'f': f84.freq=(atof(&argv[i][2])*1e6)/4; break;
                }
        }
        fprintf(stderr,"\nConfiguration info:\n-------------------\n");
        if (!(f84.config&0x3ff0))   fprintf(stderr,"Code protected\n");
        if (!(f84.config&0x8))      fprintf(stderr,"Power-up Timer enabled\n");
        if (f84.config&0x4)         fprintf(stderr,"Watchdog Timer enabled\n");
        fprintf(stderr,"%s\n\n",osc_types[f84.config&3]);
        fprintf(stderr,"---------------------------------\n");
        fprintf(stderr,"Oscillator frequency: %6.3f MHz\n",(float)f84.freq*4e-6);
        fprintf(stderr,"Cpu        frequency: %6.3f MHz\n",(float)f84.freq*1e-6);
        if (conf) exit(0);

        fprintf(stderr,"Running: Press Crtl-C to enter debug\n\n");
        setnocanon();
        fprintf(stderr,"\033[2J");
        signal(SIGINT,crtl_c);

        f84_reset(&f84,POR);
        if (n) {
                debug_status=RUNNING;
                f84_run(&f84,prog,n);
                debug_status=STEP;
        }
#ifndef SOUND
        // Setup a periodic alarm (50 Hz)
        t0.it_interval.tv_sec=0;
        t0.it_interval.tv_usec=20000;
        t0.it_value=t0.it_interval;

        act0.sa_handler=alarmh;
        act0.sa_flags=SA_RESTART;

        sigaction(SIGALRM,&act0,&act1);
        setitimer(ITIMER_REAL,&t0,&t1);

        for (;;) {
                f84_run(&f84,prog,f84.freq/50);
                // The emulation is stopped until next alarm
                do_periodic_update();
                pause();
        }
#else
        init_sound(f84.freq);
        // The real-time speed is achieved via the sound write blocking
        // No alarm needed.
        for (;;) {
                f84_run(&f84,prog,f84.freq/50);
                do_periodic_update();
        }
#endif
}
